@@ -1,8 +1,9 @@
 #ifndef PYTHONIC_INCLUDE_TYPES_NUMPY_GEXPR_HPP
 #define PYTHONIC_INCLUDE_TYPES_NUMPY_GEXPR_HPP
 
-#include "pythonic/include/utils/meta.hpp"
+#include "pythonic/include/types/numpy_iexpr.hpp"
 #include "pythonic/include/utils/array_helper.hpp"
+#include "pythonic/include/utils/meta.hpp"
 
 PYTHONIC_NS_BEGIN
 
@@ -34,6 +35,26 @@ namespace types
         count_new_axis<T0>::value + count_new_axis<T...>::value;
   };
 
+  /* helper to cast slices
+   */
+  template <size_t T, size_t Tp>
+  long recast_slice(long n)
+  {
+    return (n * (long)T) / (long)Tp;
+  }
+  template <size_t T, size_t Tp>
+  cstride_slice<1> recast_slice(normalized_slice s)
+  {
+    assert(s.step == 1 && "To change to a dtype of a different size, the last "
+                          "axis must be contiguous");
+    return {s.lower * (long)T / (long)Tp, s.upper * (long)T / (long)Tp};
+  }
+  template <size_t T, size_t Tp, long stride>
+  cstride_slice<stride> recast_slice(cstride_normalized_slice<stride> s)
+  {
+    return {s.lower * (long)T / (long)Tp, s.upper * (long)T / (long)Tp};
+  }
+
   /* helper to turn a new axis into a slice
    */
   template <class T>
@@ -58,8 +79,8 @@ namespace types
 
   template <>
   struct to_normalized_slice<none_type> {
-    using type = contiguous_normalized_slice;
-    contiguous_normalized_slice operator()(none_type);
+    using type = cstride_normalized_slice<1>;
+    type operator()(none_type);
   };
 
   /* helper to build a new shape out of a shape and a slice with new axis
@@ -73,7 +94,7 @@ namespace types
   template <size_t C>
   struct extended_slice {
     template <class E, class... S>
-    auto operator()(E &&expr, S const &... s)
+    auto operator()(E &&expr, S const &...s)
         -> decltype(std::forward<E>(expr).reshape(make_reshape<C>(
             expr,
             std::tuple<
@@ -92,7 +113,7 @@ namespace types
   template <>
   struct extended_slice<0> {
     template <class E, class... S>
-    auto operator()(E &&expr, long const &s0, S const &... s) ->
+    auto operator()(E &&expr, long const &s0, S const &...s) ->
         typename std::enable_if<
             utils::all_of<std::is_integral<S>::value...>::value,
             decltype(std::forward<E>(expr)[types::make_tuple(s0, s...)])>::type
@@ -100,7 +121,7 @@ namespace types
       return std::forward<E>(expr)[types::make_tuple(s0, s...)];
     }
     template <class E, class... S>
-    auto operator()(E &&expr, long const &s0, S const &... s) ->
+    auto operator()(E &&expr, long const &s0, S const &...s) ->
         typename std::enable_if<
             !utils::all_of<std::is_integral<S>::value...>::value,
             decltype(std::forward<E>(expr)[s0](s...))>::type
@@ -120,7 +141,7 @@ namespace types
     typename std::enable_if<
         is_slice<Sp>::value,
         numpy_gexpr<E, normalize_t<Sp>, normalize_t<S>...>>::type
-    operator()(E &&expr, Sp const &s0, S const &... s)
+    operator()(E &&expr, Sp const &s0, S const &...s)
     {
       return make_gexpr(std::forward<E>(expr), s0, s...);
     }
@@ -130,8 +151,8 @@ namespace types
         !is_slice<F>::value,
         numpy_gexpr<ndarray<typename std::decay<E>::type::dtype,
                             array<long, std::decay<E>::type::value>>,
-                    contiguous_normalized_slice, normalize_t<S>...>>::type
-    operator()(E &&expr, F const &s0, S const &... s)
+                    cstride_normalized_slice<1>, normalize_t<S>...>>::type
+    operator()(E &&expr, F const &s0, S const &...s)
     {
       return numpy_vexpr<ndarray<typename std::decay<E>::type::dtype,
                                  array<long, std::decay<E>::type::value>>,
@@ -155,8 +176,8 @@ namespace types
     static constexpr size_t value = 0;
   };
 
-  template <>
-  struct count_long<contiguous_normalized_slice> {
+  template <long stride>
+  struct count_long<cstride_normalized_slice<stride>> {
     static constexpr size_t value = 0;
   };
 
@@ -192,7 +213,7 @@ namespace types
   };
 
   template <class... S>
-  struct is_contiguous<contiguous_normalized_slice, S...> {
+  struct is_contiguous<cstride_normalized_slice<1>, S...> {
     static const bool value = true;
   };
 
@@ -213,10 +234,9 @@ namespace types
     }
 
     template <class T, class Ts>
-    auto tuple_push_head(T const &val, Ts const &vals)
-        -> decltype(tuple_push_head(
-            val, vals,
-            utils::make_index_sequence<std::tuple_size<Ts>::value>()))
+    auto
+    tuple_push_head(T const &val, Ts const &vals) -> decltype(tuple_push_head(
+        val, vals, utils::make_index_sequence<std::tuple_size<Ts>::value>()))
     {
       return tuple_push_head(
           val, vals, utils::make_index_sequence<std::tuple_size<Ts>::value>());
@@ -397,18 +417,18 @@ namespace types
       template <size_t... Is>
       numpy_gexpr<Arg, normalize_t<S>...>
       operator()(Arg arg, std::tuple<S...>, utils::index_sequence<Is...>);
-      numpy_gexpr<Arg, normalize_t<S>...> operator()(Arg arg, S const &... s);
+      numpy_gexpr<Arg, normalize_t<S>...> operator()(Arg arg, S const &...s);
     };
 
     // this specialization is in charge of merging gexpr
     template <class Arg, class... S, class... Sp>
     struct make_gexpr<numpy_gexpr<Arg, S...> const &, Sp...> {
-      auto operator()(numpy_gexpr<Arg, S...> const &arg, Sp const &... s)
-          -> decltype(
-              _make_gexpr(std::declval<Arg>(),
-                          merge_gexpr<std::tuple<S...>, std::tuple<Sp...>>{}
-                              .template run<0>(arg, std::tuple<S...>(),
-                                               std::tuple<Sp...>())))
+      auto operator()(numpy_gexpr<Arg, S...> const &arg, Sp const &...s)
+          -> decltype(_make_gexpr(
+              std::declval<Arg>(),
+              merge_gexpr<std::tuple<S...>, std::tuple<Sp...>>{}
+                  .template run<0>(arg, std::tuple<S...>(),
+                                   std::tuple<Sp...>())))
       {
         return _make_gexpr(
             arg.arg,
@@ -416,10 +436,10 @@ namespace types
                 arg, arg.slices, std::make_tuple(s...)));
       }
     };
-  }
+  } // namespace details
 
   template <class Arg, class... S>
-  auto make_gexpr(Arg &&arg, S const &... s)
+  auto make_gexpr(Arg &&arg, S const &...s)
       -> decltype(details::make_gexpr<Arg, S...>{}(std::forward<Arg>(arg),
                                                    s...));
 
@@ -504,10 +524,10 @@ namespace types
       : gexpr_shape<pshape<Tys..., long>, array<long, N - 1>> {
   };
 
-  template <class... Tys, class... oTys, class... S>
+  template <class... Tys, class... oTys, class... S, long stride>
   struct gexpr_shape<pshape<Tys...>,
                      pshape<std::integral_constant<long, 1>, oTys...>,
-                     contiguous_normalized_slice, S...>
+                     cstride_normalized_slice<stride>, S...>
       : gexpr_shape<pshape<Tys..., std::integral_constant<long, 1>>,
                     pshape<oTys...>, S...> {
   };
@@ -546,11 +566,9 @@ namespace types
     static_assert(
         utils::all_of<std::is_same<S, normalize_t<S>>::value...>::value,
         "all slices are normalized");
-    static_assert(
-        utils::all_of<(std::is_same<S, long>::value ||
-                       std::is_same<S, contiguous_normalized_slice>::value ||
-                       std::is_same<S, normalized_slice>::value)...>::value,
-        "all slices are valid");
+    static_assert(utils::all_of<(std::is_same<S, long>::value ||
+                                 is_normalized_slice<S>::value)...>::value,
+                  "all slices are valid");
     static_assert(std::decay<Arg>::type::value >= sizeof...(S),
                   "slicing respects array shape");
 
@@ -572,6 +590,11 @@ namespace types
     static constexpr size_t value =
         std::remove_reference<Arg>::type::value - count_long<S...>::value;
 
+    using last_arg_stride_t =
+        decltype(std::declval<Arg>().template strides<sizeof...(S) - 1>());
+    using last_slice_t =
+        typename std::tuple_element<sizeof...(S) - 1, std::tuple<S...>>::type;
+
     // It is not possible to vectorize everything. We only vectorize if the
     // last dimension is contiguous, which happens if
     // 1. Arg is an ndarray (this is too strict)
@@ -580,18 +603,19 @@ namespace types
     static const bool is_vectorizable =
         std::remove_reference<Arg>::type::is_vectorizable &&
         (sizeof...(S) < std::remove_reference<Arg>::type::value ||
-         std::is_same<contiguous_normalized_slice,
-                      typename std::tuple_element<
-                          sizeof...(S)-1, std::tuple<S...>>::type>::value);
+         std::is_same<cstride_normalized_slice<1>, last_slice_t>::value);
+    static const bool is_flat =
+        std::remove_reference<Arg>::type::is_flat && value == 1 &&
+        utils::all_of<
+            std::is_same<cstride_normalized_slice<1>, S>::value...>::value;
     static const bool is_strided =
         std::remove_reference<Arg>::type::is_strided ||
-        (((sizeof...(S)-count_long<S...>::value) == value) &&
-         !std::is_same<contiguous_normalized_slice,
-                       typename std::tuple_element<
-                           sizeof...(S)-1, std::tuple<S...>>::type>::value);
+        (((sizeof...(S) - count_long<S...>::value) == value) &&
+         !std::is_same<cstride_normalized_slice<1>, last_slice_t>::value);
 
-    using value_type = typename std::decay<decltype(
-        numpy_iexpr_helper<value>::get(std::declval<numpy_gexpr>(), 1))>::type;
+    using value_type =
+        typename std::decay<decltype(numpy_iexpr_helper<value>::get(
+            std::declval<numpy_gexpr>(), 1))>::type;
 
     using iterator =
         typename std::conditional<is_strided || value != 1,
@@ -610,7 +634,21 @@ namespace types
 
     shape_t _shape;
     dtype *buffer;
-    array<long, value> _strides;
+
+    static constexpr types::pshape<std::integral_constant<long, 1>>
+        last_stride(cstride_normalized_slice<1>);
+    template <long stride>
+    static constexpr types::pshape<std::integral_constant<long, stride>>
+        last_stride(cstride_normalized_slice<stride>);
+    static constexpr types::array<long, 1> last_stride(...);
+
+    sutils::concat_t<types::array<long, value - 1>,
+                     typename std::conditional<
+                         sizeof...(S) == std::decay<Arg>::type::value,
+                         decltype(last_stride(std::declval<last_slice_t>())),
+                         types::array<long, 1>>::type>
+        _strides; // strides
+
     template <size_t I>
     auto shape() const -> decltype(std::get<I>(_shape))
     {
@@ -632,17 +670,11 @@ namespace types
     numpy_gexpr(numpy_gexpr<Argp, S...> const &other);
 
     template <size_t J, class Slice>
-    typename std::enable_if<
-        std::is_same<Slice, normalized_slice>::value ||
-            std::is_same<Slice, contiguous_normalized_slice>::value,
-        void>::type
+    typename std::enable_if<is_normalized_slice<Slice>::value, void>::type
     init_shape(Slice const &s, utils::int_<1>, utils::int_<J>);
 
     template <size_t I, size_t J, class Slice>
-    typename std::enable_if<
-        std::is_same<Slice, normalized_slice>::value ||
-            std::is_same<Slice, contiguous_normalized_slice>::value,
-        void>::type
+    typename std::enable_if<is_normalized_slice<Slice>::value, void>::type
     init_shape(Slice const &s, utils::int_<I>, utils::int_<J>);
 
     template <size_t J>
@@ -688,7 +720,7 @@ namespace types
     // have to
     // be increased
     numpy_gexpr(Arg const &arg, std::tuple<S const &...> const &values);
-    numpy_gexpr(Arg const &arg, S const &... s);
+    numpy_gexpr(Arg const &arg, S const &...s);
 
   public:
     template <class Argp, class... Sp>
@@ -767,8 +799,8 @@ namespace types
     iterator begin();
     iterator end();
 
-    auto fast(long i) const
-        & -> decltype(numpy_iexpr_helper<value>::get(*this, i))
+    auto fast(long i) const & -> decltype(numpy_iexpr_helper<value>::get(*this,
+                                                                         i))
     {
       return numpy_iexpr_helper<value>::get(*this, i);
     }
@@ -810,27 +842,27 @@ namespace types
 #endif
 
     template <class... Sp>
-    auto operator()(Sp const &... s) const -> decltype(make_gexpr(*this, s...));
+    auto operator()(Sp const &...s) const -> decltype(make_gexpr(*this, s...));
 
     template <class Sp>
     auto operator[](Sp const &s) const -> typename std::enable_if<
         is_slice<Sp>::value, decltype(make_gexpr(*this, (s.lower, s)))>::type;
 
     template <size_t M>
-    auto fast(array<long, M> const &indices) const
-        & -> decltype(nget<M - 1>().fast(*this, indices));
+    auto fast(array<long, M> const &indices)
+        const & -> decltype(nget<M - 1>().fast(*this, indices));
 
     template <size_t M>
-        auto fast(array<long, M> const &indices) &&
-        -> decltype(nget<M - 1>().fast(std::move(*this), indices));
+    auto fast(array<long, M> const &indices) && -> decltype(nget<M - 1>().fast(
+        std::move(*this), indices));
 
     template <size_t M>
-    auto operator[](array<long, M> const &indices) const
-        & -> decltype(nget<M - 1>()(*this, indices));
+    auto operator[](array<long, M> const &indices)
+        const & -> decltype(nget<M - 1>()(*this, indices));
 
     template <size_t M>
-        auto operator[](array<long, M> const &indices) &&
-        -> decltype(nget<M - 1>()(std::move(*this), indices));
+    auto operator[](array<long, M> const &indices) && -> decltype(nget<M - 1>()(
+        std::move(*this), indices));
 
     template <class F> // indexing through an array of indices -- a view
     typename std::enable_if<is_numexpr_arg<F>::value &&
@@ -873,14 +905,40 @@ namespace types
 
     explicit operator bool() const;
 
+    dtype *data()
+    {
+      return buffer;
+    }
+    const dtype *data() const
+    {
+      return buffer;
+    }
+
     long flat_size() const;
     long size() const;
     ndarray<dtype, shape_t> copy() const
     {
       return {*this};
     }
+
+    template <class Tp, size_t... Is>
+    auto recast(utils::index_sequence<Is...>) -> decltype(make_gexpr(
+        arg.template recast<Tp>(),
+        recast_slice<sizeof(dtype), sizeof(Tp)>(std::get<Is>(slices))...))
+    {
+      return make_gexpr(
+          arg.template recast<Tp>(),
+          recast_slice<sizeof(dtype), sizeof(Tp)>(std::get<Is>(slices))...);
+    }
+
+    template <class Tp>
+    auto recast()
+        -> decltype(recast<Tp>(utils::make_index_sequence<sizeof...(S)>()))
+    {
+      return recast<Tp>(utils::make_index_sequence<sizeof...(S)>());
+    }
   };
-}
+} // namespace types
 
 template <class E, class... S>
 struct assignable_noescape<types::numpy_gexpr<E, S...>> {
@@ -925,9 +983,8 @@ struct __combined<pythonic::types::numpy_gexpr<Arg, S...>,
   using type =
       pythonic::types::ndarray <
       typename __combined<typename t0::dtype, typename t1::dtype>::type,
-        pythonic::types::array < long,
-        t0::value < t1::value ? t1::value : t0::value >>
-      ;
+        pythonic::types::array<long,
+                               t0::value<t1::value ? t1::value : t0::value>>;
 };
 
 template <class Arg, class... S, class O>
