@@ -22,13 +22,17 @@ from pythran.version import __version__
 from pythran.utils import cxxid
 import pythran.frontend as frontend
 
-from datetime import datetime
 from distutils.errors import CompileError
 from distutils import sysconfig
-from numpy.distutils.core import setup
+try:
+    # `numpy.distutils is deprecated, may not be present, or broken
+    from numpy.distutils.core import setup
+except Exception:
+    from distutils.core import setup
 
 from tempfile import mkdtemp, NamedTemporaryFile
 import gast as ast
+import importlib
 import logging
 import os.path
 import shutil
@@ -178,8 +182,7 @@ def generate_cxx(module_name, code, specs=None, optimizations=None,
         else:
             code_bytes = code.encode('ascii', 'ignore')
         metainfo = {'hash': hashlib.sha256(code_bytes).hexdigest(),
-                    'version': __version__,
-                    'date': datetime.now()}
+                    'version': __version__}
 
         mod = PythonModule(module_name, docstrings, metainfo)
         mod.add_to_includes(
@@ -429,6 +432,23 @@ def compile_pythrancode(module_name, pythrancode, specs=None,
     return output_file
 
 
+def import_pythrancode(pythrancode, **kwargs):
+    # It should be safe to delete the library once it's loaded in memory.
+    digest = hashlib.sha256(pythrancode.encode()).hexdigest()
+    module_name = "pythranized_{}".format(digest)
+    tmpfile = None
+    try:
+        tmpfile = compile_pythrancode(module_name, pythrancode, **kwargs)
+        spec = importlib.util.spec_from_file_location(module_name, tmpfile)
+        module = importlib.util.module_from_spec(spec)
+        sys.modules[module_name] = module
+        spec.loader.exec_module(module)
+        return module
+    finally:
+        if tmpfile is not None:
+            os.remove(tmpfile)
+
+
 def compile_pythranfile(file_path, output_file=None, module_name=None,
                         cpponly=False, pyonly=False, report_times=False, **kwargs):
     """
@@ -488,6 +508,11 @@ def compile_pythranfile(file_path, output_file=None, module_name=None,
     return output_file
 
 
+def import_pythranfile(pythranpath, **kwargs):
+    with open(pythranpath) as fd:
+        return import_pythrancode(fd.read(), **kwargs)
+
+
 def test_compile():
     '''Simple passthrough compile test.
     May raises CompileError Exception.
